#include "PlayState.h"

#include <iostream>
#include <vector>

#include <RedBox/ResourceManager.h>
#include <RedBox/ResourcePathHandler.h>
#include <RedBox/AudioEngine.h>
#include <RedBox/MusicEngine.h>
#include <RedBox/SoundEngine.h>
#include <RedBox/SoundFX.h>
#include <RedBox/BackgroundMusic.h>
#include <RedBox/Random.h>
#include <RedBox/RedBoxEngine.h>

#include <RedBox/Sprite.h>
#include <RedBox/InputManager.h>
#include <RedBox/Keyboard.h>
#include <RedBox/TimeHelper.h>
#include <RedBox/RedBoxEngine.h>
#include <RedBox/Color.h>
#include <RedBox/SpriteFactory.h>
#include <RedBox/RenderStep.h>
#include <RedBox/AnimationParameters.h>
#include <RedBox/Vec2.h>
#include <RedBox/SpriteEmitter.h>
#include <RedBox/Font.h>
#include <RedBox/GraphicString.h>

#include <stdint.h>

using namespace RedBox;

PlayState::PlayState(): RedBox::State(), nbUdpates(0), lastSecond(0.0),
nbRenders(0), lastSecond2(0.0) {
	setBackgroundColor(Color::BLACK);
	ResourceManager::loadTexture(ResourcePathHandler::getResourcePathFor("redbox.png"), "custom");
	ResourceManager::loadTexture(ResourcePathHandler::getResourcePathFor("flash.png"), "orb");
	ResourceManager::loadTexture(ResourcePathHandler::getResourcePathFor("francis-bacon.trans.png"), "bacon");
	ResourceManager::loadTexture(ResourcePathHandler::getResourcePathFor("floor.png"), "floor");
	ResourceManager::loadSound(ResourcePathHandler::getResourcePathFor("hey.wav"), "hey");
	ResourceManager::loadSound(ResourcePathHandler::getResourcePathFor("jump.wav"), "jump");
	ResourceManager::loadSound(ResourcePathHandler::getResourcePathFor("explosion.wav"), "explosion");

	Font* fearFont = ResourceManager::loadFont("fear", ResourcePathHandler::getResourcePathFor("fear.ttf"));

	GraphicString* myText = new GraphicString(fearFont);
	myText->setText("Francis Bacon");
	myText->setPixelSize(87);
	myText->setPosition(50, 100);
	myText->setColor(200, 0, 0, 255);
	myText->setZ(40);
	addRenderable(myText);

	//Sprite* test3 = new Sprite("animated", 32, 32, 4);
	//std::vector<unsigned int> tmpVec(1, 0);
	//tmpVec[0] = 2;
	//test3->getMainRenderStep()->getRenderInfo().addAnimation("lol", AnimationParameters(tmpVec, 0.05, -1));
	//test3->getMainRenderStep()->playAnimation("lol");
	//test3->setPosition(250.0f, 400.0f);
	//addRenderable(test3);

	//AudioEngine::getSoundEngine()->getSoundFX("hey", false)->play(-1);

	ResourceManager::loadMusic(ResourcePathHandler::getResourcePathFor("intense.wav"), "music");

	AudioEngine::getMusicEngine()->getBackgroundMusic("music", false)->play(-1, 4.5);
	
	shape = new Sprite("bacon");
	shape->setZ(0);
	shape->setAcceleration(Vec2(0.0f, 900.0f));
	shape->setMaxVelocityY(600.0f);
	shape->setDrag(Vec2(MOVE_ACCEL, 0.0f));
	addRenderable(shape);
	//addRenderable(new Sprite(*shape));
	shape->setPosition(100.0f, 200.0f);

	floor = new Sprite("floor");
	floor->setIsStatic(true);
	floor->setPosition(0.0f, 448.0f);
	addRenderable(floor);

	Keyboard::getDefault()->keyHold.connect(this, &PlayState::onKeyHold);
	Keyboard::getDefault()->keyPress.connect(this, &PlayState::onKeyPress);
	Keyboard::getDefault()->keyRelease.connect(this, &PlayState::onKeyRelease);
	Pointer::getDefault()->buttonPress.connect(this, &PlayState::onMousePress);
	Pointer::getDefault()->buttonRelease.connect(this, &PlayState::onMouseRelease);
	//InputManager::getInstance().getPointer()->buttonHold.connect(this, &PlayState::onMouseHold);
	//Pointer::getDefault()->move.connect(this, &PlayState::onMouseMove);
	exploder.setInterval(2.0);
	exploder.tick.connect(this, &PlayState::randomExplosion);
	exploder.start();
}

void PlayState::onKeyPress(RedBox::KeySignalData data) {
	if(data.key == Key::S) {
		camera.shake();
	} else if(data.key == Key::W) {
		TimeHelper::getInstance().sleep(1.0);
	} else if(data.key == Key::R) {
		shape->setPosition(100.0f, 200.0f);
	} else if(data.key == Key::RIGHT) {
		shape->setAccelerationX(shape->getAcceleration().getX() + MOVE_ACCEL);
	} else if(data.key == Key::LEFT) {
		shape->setAccelerationX(shape->getAcceleration().getX() - MOVE_ACCEL);
	}

}

void PlayState::onKeyHold(RedBox::KeySignalData data) {
}

void PlayState::onKeyRelease(RedBox::KeySignalData data) {
	if(data.key == Key::RIGHT) {
		shape->setAccelerationX(shape->getAcceleration().getX() - MOVE_ACCEL);
	} else if(data.key == Key::LEFT) {
		shape->setAccelerationX(shape->getAcceleration().getX() + MOVE_ACCEL);
	}
}

void PlayState::onMousePress(RedBox::PointerButtonSignalData data) {
	emitExplosion(data.pointerState.getCursorState().getPosition().getX() - 16.0f,
				  data.pointerState.getCursorState().getPosition().getY() - 16.0f);
}

void PlayState::onMouseMove(RedBox::PointerMoveSignalData data) {
}

void PlayState::onMouseRelease(RedBox::PointerButtonSignalData data) {
}

void PlayState::onMouseHold(RedBox::PointerButtonSignalData data) {
}

void PlayState::update() {
	State::update();

	if(shape->collide(floor).first) {
		if(Keyboard::getDefault()->getState().isKeyPressed(Key::UP)) {
			shape->setVelocity(Vec2(shape->getVelocity().getX(), JUMP_VEL));
			AudioEngine::getSoundEngine()->getSoundFX("jump", false)->play();
		}
	}

	Color tmp(getBackgroundColor());
	if(tmp.getBlue() == 255) {
		tmp.setBlue(0);
	} else {
		tmp.setBlue(tmp.getBlue() + 1);
	}
	if(tmp.getRed() == 255) {
		tmp.setRed(0);
	} else {
		tmp.setRed(tmp.getRed() + 3);
	}
	if(tmp.getGreen() == 255) {
		tmp.setGreen(0);
	} else {
		tmp.setGreen(tmp.getGreen() + 5);
	}
	setBackgroundColor(tmp);

	++nbUdpates;
	if(RedBox::TimeHelper::getInstance().getSinceStartComplete() > lastSecond + 1.0) {
		lastSecond += 1.0;
		std::cout << "up: " << nbUdpates << " | " << RedBox::TimeHelper::getInstance().getSinceStartComplete() << " | " << RedBox::RedBoxEngine::getUpdatesPerSecond() << std::endl;
		nbUdpates = 0;
	}
}

void PlayState::render() {
	State::render();
	++nbRenders;
	if(RedBox::TimeHelper::getInstance().getSinceStartComplete() > lastSecond2 + 1.0) {
		lastSecond2 += 1.0;
		std::cout << "re: " << nbRenders << " | " << RedBox::TimeHelper::getInstance().getSinceStartComplete() << " | " << RedBox::RedBoxEngine::getMinFps() << std::endl;
		nbRenders = 0;
	}
}

void PlayState::emitExplosion(float x, float y) {
	Sprite* particle = new Sprite("orb", 32, 32, 2);
	particle->getMainRenderStep()->getRenderInfo().addAnimation("lol", 0.05, -1, 2, 0, 1);
	particle->getMainRenderStep()->playAnimation("lol");
	SpriteEmitter* emitter = new SpriteEmitter(particle);
	emitter->setForce(500.0f);
	emitter->setForceVariance(1000.0f);
	emitter->setAngle(0.0f);
	emitter->setAngleVariance(360.0f);
	emitter->setAcceleration(Vec2(0.0f, 900.0f));
	emitter->setEmitRate(2000.0);
	emitter->setNbMaxParticles(100.0);
	emitter->setNbParticlesToShoot(100);
	emitter->setLifeSpan(-1.0);
	emitter->activate();
	emitter->setLifePhase(ParticlePhase(1.0, 0.2, 0.0f, 0.0f));
	emitter->setPosition(x, y);
	addRenderable(emitter);
	AudioEngine::getSoundEngine()->getSoundFX("explosion", false)->play();
	camera.shake();
}

void PlayState::randomExplosion() {
	emitExplosion(Random::getRandomFloat(0.0f, 640.0f), Random::getRandomFloat(0.0f, 480.0f));
}
